// Copyright 2005-2021 The Mumble Developers. All rights reserved.
// Use of this source code is governed by a BSD-style license
// that can be found in the LICENSE file at the root of the
// Mumble source tree or at <https://www.mumble.info/LICENSE>.

#include <winver.h>

#ifndef DEBUG
#define VER_DEBUG                   0L
#else
#define VER_DEBUG                   VS_FF_DEBUG
#endif

#ifdef SNAPSHOT_BUILD
#define	VER_RELEASE	VS_FF_SPECIALBUILD|VS_FF_PRERELEASE
#else
#define VER_RELEASE 0L
#endif

VS_VERSION_INFO VERSIONINFO
	FILEVERSION		1,3,4,0
	PRODUCTVERSION	1,3,4,0
	FILEFLAGSMASK	VS_FFI_FILEFLAGSMASK
	FILEFLAGS		(VER_DEBUG|VER_RELEASE)
	FILEOS			VOS_NT_WINDOWS32
	FILETYPE		VFT_DLL
	FILESUBTYPE		0L
	BEGIN
		BLOCK "StringFileInfo"
		BEGIN
			BLOCK "040904E4"
			BEGIN
				VALUE "CompanyName", "The Mumble Developers"
				VALUE "FileDescription", "Mumble - Low-latency VoIP client"
				VALUE "FileVersion", "1.3.4"
				VALUE "ProductVersion", "1.3.4"
				VALUE "LegalCopyright", "Copyright (c) 2005-2021 The Mumble Developers"
				VALUE "OriginalFilename", "mumble_ol_x64.dll"
				VALUE "ProductName", "Mumble"
#ifdef SNAPSHOT_BUILD
				VALUE "SpecialBuild", "Snapshot development release"
#endif
			END
		END
		BLOCK "VarFileInfo"
		BEGIN
			VALUE "Translation", 0x409, 1252
		END
	END
